/*
	WordWrap.cmd
	WordWrap v1.4 (c) 2001,2003 by Don K Eitner

	Reformats text files and reflows them to a line length you specify.
	Also can output as basic HTML for use with web browsers.

	This code is neither supported nor under warranty.  Feel free to
	examine and modify this script for your own purposes.  See the
	included readme.txt for additional information.
*/

/* Initialize Rexx utility functions */
CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
CALL SysLoadFuncs

/* Setup ANSI codes for display properties */
CALL AnsiSetup

/* Set initial global variables */
PARSE ARG CmdLine
Outline = ''
OutFile = ''

/* Find out what the user wants us to do */
CALL ParseInput

/* Now that we know what the user wants us to do, let's do it */

SELECT
	WHEN Margin < 0 THEN   /* We cannot do this */
		DO
			SAY LRed||'Line length must be a whole number greater than or equal to zero.'||Non
			EXIT
		END
	WHEN Margin = 0 THEN   /* Only reformat, don't wrap lines */
		DO
			CALL Prep
			CALL Reformat
		END
	OTHERWISE   /* Reformat and wrap lines */
		DO
			CALL Prep
			CALL Reformat
			CALL Wrap
		END
END

CALL CleanUp

EXIT


AnsiSetup:

	Esc=d2c(27)d2c(91)

	/* Attributes */
	Hig=Esc'1m'
	Non=Esc'0m'

	/* Foreground Colors */
	Bla=Esc'30m'
	Red=Esc'31m'
	Gre=Esc'32m'
	Bro=Esc'33m'
	Blu=Esc'34m'
	Pur=Esc'35m'
	Tur=Esc'36m'
	Whi=Esc'37m'
	LBla=Hig||Bla
	LRed=Hig||Red
	LGre=Hig||Gre
	LBro=Hig||Bro
	LBlu=Hig||Blu
	LPur=Hig||Pur
	LTur=Hig||Tur
	LWhi=Hig||Whi

	/* Background Colors */
	BBla=Esc'40m'
	BRed=Esc'41m'
	BGre=Esc'42m'
	BBro=Esc'43m'
	BBlu=Esc'44m'
	BCya=Esc'45m'
	BTur=Esc'46m'
	BWhi=Esc'47m'

RETURN


ParseInput:

	IF CmdLine = '' THEN
		CALL GiveHelp

	DO WHILE CmdLine <> ''
		PARSE VAR CmdLine CL1 CmdLine
		IF Left(CL1, 1) = '/' | Left(CL1, 1) = '-' THEN
			DO
				CL1 = Translate(CL1)
	               SELECT
					WHEN Substr(CL1, 2) = 'M' THEN   /* set output line length */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							Margin = CL1
							IF (DATATYPE(Margin) <> 'NUM') THEN
								DO
									SAY LRed||'You must provide a line length (in # of characters).'||Non
									EXIT
								END
						END
					WHEN Substr(CL1, 2) = 'F' THEN   /* get output format */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							OutForm = Translate(CL1)
						END
					WHEN Substr(CL1, 2) = 'E' THEN   /* select end-of-line character */
						DO
							PARSE VAR CmdLine CL1 CmdLine
							EOL = Translate(CL1)
						END
					WHEN Substr(CL1, 2) = 'I' THEN   /* get input file name */
						DO
							IF Left(CmdLine, 1) = '"' THEN
								DO
									PARSE VAR CmdLine WITH '"' CL1 '"' CmdLine
									Infile = CL1
								END
							ELSE
								DO
									PARSE VAR CmdLine CL1 CmdLine
									Infile = CL1
								END
						END
					WHEN Substr(CL1, 2) = 'O' THEN   /* get output file name */
						DO
							IF Left(CmdLine, 1) = '"' THEN
								DO
									PARSE VAR CmdLine WITH '"' CL1 '"' CmdLine
									OutFile = CL1
								END
							ELSE
								DO
									PARSE VAR CmdLine CL1 CmdLine
									OutFile = CL1
								END
						END
					OTHERWISE
						CALL GiveHelp
				END
			END
		ELSE
			DO
				CALL GiveHelp
				EXIT
			END
	END

	SELECT
		WHEN EOL = 'UNIX' | EOL = 'LF' | EOL = '0A' THEN
			DO
				EOL2 = 'UNIX'
				EOL = '0a'x
			END
		WHEN EOL = 'DOS' | EOL = 'CRLF' | EOL = '0D0A' THEN
			DO
				EOL2 = 'DOS'
				EOL = '0d0a'x
			END
		WHEN EOL = 'MAC' | EOL = 'CR' | EOL = '0D' THEN
			DO
				EOL2 = 'MAC'
				EOL = '0d'x
			END
		OTHERWISE
			EOL = '0d0a'x
	END

RETURN


GiveHelp:

	CALL SysCls
	SAY LTur||'WordWrap v1.4, (c) 2003 by Don Eitner'||Non
	SAY ''
	SAY LGre||'Usage:'||Non
	SAY '  WordWrap -M Margin -I InFile -O OutFile [options]'
	SAY ''
	SAY ' Margin must be an integer.  Margin of 0 creates single-line paragraphs with'
	SAY ' no word wrapping; good for importing to a word processor program.'
	SAY ''
	SAY ' InFile and OutFile must be enclosed with quotation marks if the filenames'
	SAY ' contain spaces.'
	SAY ''
	SAY LGre||'Available Options:'||Non
	SAY '  -F = specify output format as TEXT or HTML'
	SAY LWhi||'       Default is TEXT if not given.'||Non
	SAY '  -E = specify output EOL character'
	SAY '       To output for OS/2, DOS or Win, use /E DOS or /E CRLF or /E 0d0a'
	SAY '       To output for UNIX, use /E UNIX or /E LF or /E 0a'
	SAY '       To output for Macintosh, use /E MAC or /E CR or /E 0d'
	SAY LWhi||'       Default is DOS if not given.'||Non
	SAY '  -? = show this help screen'

EXIT


Prep:

	/* Prepare work files */
	X = Lastpos('.', Infile)
	IF X > 0 THEN
		DO
			Tempfile = Substr(Infile, 1, X-1)||'.tmp'
			IF OutFile = '' THEN
				Outfile = Substr(Infile, 1, X-1)||'.out'
		END
	ELSE
		DO
			Tempfile = Infile||'.tmp'
			IF OutFile = '' THEN
				Outfile = Infile||'.out'
		END

	/* Delete temporary file if it exists so we don't keep any old junk */
	rc = Stream(Tempfile, 'c', 'QUERY EXISTS')
	IF rc <> '' THEN
		CALL SysFileDelete Tempfile

	/* Backup original file to temporary directory for safety, be sure it exists first */
	rc = Stream(Infile, 'c', 'QUERY EXISTS')
	IF rc = '' THEN
		DO
			SAY LRed||'Input file'||Non Infile LRed||'was not found.'||Non
			EXIT
		END
	ELSE
		DO
			SAY LBlu||'*' LTur||'Backing up' LBro||Infile LTur||'to /temp directory'||Non
			CALL SysMkDir 'temp'
			'@copy "'||Infile||'" temp'
		END

RETURN


Reformat:

	/* Open the input file, report any errors */
	rc = Stream(Infile, 'c', 'OPEN')
	IF rc <> 'READY:' THEN
		DO
			SAY 'Input file cannot be opened. Error was' rc
			EXIT
		END
	ELSE
		/* Remove extraneous spaces from input file and concatenate lines */
		SAY LBlu||'*' LTur||'Reformatting' LBro||Infile LTur||'to' LBro||Tempfile||Non

	/* Open the temporary output file, report any errors */
	rc = Stream(Tempfile, 'c', 'OPEN')
	IF rc <> 'READY:' THEN
		DO
			SAY 'Temporary file cannot be opened. Error was' rc
			EXIT
		END

	SELECT
		WHEN OutForm = 'HTML' THEN
			DO
				HTMLHead = '<html>'||EOL||'<head>'||EOL||'<title>'||Outfile||'</title>'||EOL||'<body>'||EOL||EOL
				CALL CharOut Tempfile, HTMLHead
			END
		OTHERWISE
			NOP
	END

	/* Read in the input file line by line */
	DO WHILE Lines(Infile) > 0
		Inline = Linein(Infile,,1)

		/* Convert tabs in input line to spaces in output line */
		DO FOREVER
			X = Pos('09'x, Inline)
			IF X > 0 THEN
				DO
					A = substr(Inline, 1, X - 1)
					B = substr(Inline, X + 1)
					Inline = A||' '||B
				END
			ELSE
				LEAVE
		END

		/* If input line already contains basic HTML code, strip it out */
		IF Translate(Left(Inline, 3)) = '<P>' THEN
			Inline = Substr(Inline, 4)

		Inline = Space(Inline)
		Outline = Outline||' '||Inline
		DROP Inline

		X = Pos('  ',Outline)
		IF X > 0 THEN
			DO
				A = Substr(Outline, 1, X - 1)
				A = Space(A)
				Outline = Substr(Outline, X + 2)
				SELECT
					WHEN OutForm = 'HTML' THEN
						A = '<p>'||A||'</p>'||EOL||EOL
					OTHERWISE
						A = A||EOL||EOL
				END
				CALL CharOut Tempfile, A
			END
	END

	/* Close the input file */
	rc = Stream(Infile, 'c', 'CLOSE')

	Outline = Space(Outline)

	SELECT
		WHEN OutForm = 'HTML' THEN
			DO
				HTMLFoot = EOL||EOL||'</body>'||EOL||'</html>'
				Outline = '<p>'||Outline||'</p>'||HTMLFoot
				CALL CharOut Tempfile, Outline
			END
		OTHERWISE
			CALL CharOut Tempfile, Outline
	END

	/* Close the temporary input file */
	rc = Stream(Tempfile, 'c', 'CLOSE')

RETURN


Wrap:

	/* Delete output file if it exists so we don't keep any old junk */
	rc = Stream(Outfile, 'c', 'QUERY EXISTS')
	IF rc <> '' THEN
		CALL SysFileDelete Outfile

	/* Open the temporary input file, report any errors */
	rc = Stream(Tempfile, 'c', 'OPEN')
	IF rc <> 'READY:' THEN
		DO
			SAY 'Temporary file cannot be opened. Error was' rc
			EXIT
		END

	/* Open the output file, report any errors */
	rc = Stream(Outfile, 'c', 'OPEN')
	IF rc <> 'READY:' THEN
		DO
			SAY 'Output file cannot be opened. Error was' rc
			EXIT
		END

	OutLine = ''

	/* Wrap single line paragraphs to specified margin width */
	SAY LBlu||'*' LTur||'Word wrapping' LBro||Tempfile LTur||'to' LBro||Outfile||Non
	DO WHILE Lines(Tempfile) > 0
		Inline = Linein(Tempfile,,1)
		IF Inline = '' THEN
			ITERATE
		Outline = ''
		IF Length(Inline) > Margin THEN
			DO
				First = 1
				DO UNTIL First >= Length(Inline)
					A = Substr(Inline, First, Margin + 1)
					Blank = Lastpos(' ', A)
					IF Blank = 0 THEN
						DO
							Inline = Inline||' '
							Blank = Pos(' ', Inline, First) - First + 1
						END
					Last = Blank - 1
					A = Substr(Inline, First, Last)
					A = Space(A)
					OutLine = OutLine||A||EOL
					First = First + Blank
				END
				OutLine = OutLine||EOL
			END
		ELSE
			OutLine = OutLine||InLine||EOL||EOL
		CALL CharOut Outfile, Outline
	END

	/* Close the temporary input file */
	rc = Stream(Tempfile, 'c', 'CLOSE')

	/* Close the output file */
	rc = Stream(Outfile, 'c', 'CLOSE')

RETURN


CleanUp:

	/* Clean up after ourselves */
	IF Margin = 0 THEN
		DO
			SAY LBlu||'*' LTur||'Renaming' LBro||Tempfile LTur||'to' LBro||Outfile||Non
			CALL SysFileDelete OutFile
			'@ren "'||TempFile||'" "'||OutFile||'"'
		END

	SAY LBlu||'*' LTur||'Cleaning up directory'||Non
	CALL SysFileDelete Tempfile

RETURN

